@@ -52,32 +52,42 @@ class DaprInferenceClientConfig:
 class HFInferenceClientConfig(BaseModel):
     model: Optional[str] = Field(
         None,
-        description="Model ID on Hugging Face Hub or URL to a deployed Inference Endpoint. Defaults to a recommended model if not provided.",
+        description="Model ID on Hugging Face Hub or a URL to a deployed endpoint. If not set, a recommended model may be chosen by your wrapper.",
     )
-    api_key: Optional[Union[str, bool]] = Field(
-        None,
-        description="Hugging Face API key for authentication. Defaults to the locally saved token. Pass False to skip token.",
+    hf_provider: Optional[str] = Field(
+        "auto",
+        description="Inference provider to use. Defaults to automatic selection based on available providers. Ignored if a custom endpoint URL is provided.",
     )
     token: Optional[Union[str, bool]] = Field(
         None,
-        description="Alias for api_key. Defaults to the locally saved token. Pass False to avoid sending the token.",
+        description="Hugging Face access token for authentication. If None, uses the locally saved token. Set to False to skip sending a token. Mutually exclusive with api_key.",
+    )
+    api_key: Optional[Union[str, bool]] = Field(
+        None, description="Alias for token. Use only one of token or api_key."
     )
     base_url: Optional[str] = Field(
         None,
-        description="Base URL to run inference. Cannot be used if model is set. Defaults to None.",
+        description="Custom endpoint URL for inference. Used for private deployments or TGI endpoints. Cannot be set if 'model' is a Hub ID.",
     )
     timeout: Optional[float] = Field(
         None,
-        description="Maximum time in seconds to wait for a server response. Defaults to None, meaning it will wait indefinitely.",
+        description="Maximum seconds to wait for a response. If None, waits indefinitely. Useful for slow model loading.",
     )
     headers: Optional[Dict[str, str]] = Field(
-        None,
-        description="Additional headers to send to the server. Overrides default headers such as authorization and user-agent.",
+        default_factory=dict,
+        description="Extra HTTP headers to send with requests. Overrides defaults like authorization and user-agent.",
     )
     cookies: Optional[Dict[str, str]] = Field(
-        None, description="Additional cookies to send with the request."
+        default_factory=dict, description="Extra cookies to send with requests."
+    )
+    proxies: Optional[Any] = Field(
+        None,
+        description="Proxy settings for HTTP requests. Use standard requests format.",
+    )
+    bill_to: Optional[str] = Field(
+        None,
+        description="Billing account for requests. Only used for enterprise/organization billing.",
     )
-    proxies: Optional[Any] = Field(None, description="Proxies to use for the request.")
 
     @field_validator("*", mode="before")
     @classmethod
