@@ -4,33 +4,22 @@ import inspect
 import json
 import logging
 import sys
-import time
 import uuid
 from typing import Any, Callable, Dict, List, Optional, TypeVar, Union
 
-from pydantic import BaseModel, ConfigDict, Field
-
-from durabletask import task as dtask
-
-from dapr.clients import DaprClient
-from dapr.clients.grpc._request import (
-    TransactionOperationType,
-    TransactionalStateOperation,
-)
-from dapr.clients.grpc._response import StateResponse
-from dapr.clients.grpc._state import Concurrency, Consistency, StateOptions
 from dapr.ext.workflow import (
     DaprWorkflowClient,
     WorkflowActivityContext,
     WorkflowRuntime,
 )
 from dapr.ext.workflow.workflow_state import WorkflowState
+from durabletask import task as dtask
+from pydantic import BaseModel, ConfigDict, Field
 
+from dapr_agents.agents.base import ChatClientBase
 from dapr_agents.types.workflow import DaprWorkflowStatus
 from dapr_agents.workflow.task import WorkflowTask
-from dapr_agents.workflow.utils import get_decorated_methods
-from dapr_agents.agents.base import ChatClientType
-from dapr_agents.llm.openai import OpenAIChatClient
+from dapr_agents.workflow.utils.core import get_decorated_methods, is_pydantic_model
 
 logger = logging.getLogger(__name__)
 
@@ -42,9 +31,9 @@ class WorkflowApp(BaseModel):
     A Pydantic-based class to encapsulate a Dapr Workflow runtime and manage workflows and tasks.
     """
 
-    llm: ChatClientType = Field(
-        default_factory=OpenAIChatClient,
-        description="The default LLM client for all LLM-based tasks.",
+    llm: Optional[ChatClientBase] = Field(
+        default=None,
+        description="The default LLM client for tasks that explicitly require an LLM but don't specify one (optional).",
     )
     # TODO: I think this should be within the wf client or wf runtime...?
     timeout: int = Field(
@@ -62,9 +51,6 @@ class WorkflowApp(BaseModel):
     wf_client: Optional[DaprWorkflowClient] = Field(
         default=None, init=False, description="Workflow client instance."
     )
-    client: Optional[DaprClient] = Field(
-        default=None, init=False, description="Dapr client instance."
-    )
     tasks: Dict[str, Callable] = Field(
         default_factory=dict, init=False, description="Dictionary of registered tasks."
     )
@@ -84,7 +70,6 @@ class WorkflowApp(BaseModel):
         self.wf_runtime = WorkflowRuntime()
         self.wf_runtime_is_running = False
         self.wf_client = DaprWorkflowClient()
-        self.client = DaprClient()
         logger.info("WorkflowApp initialized; discovering tasks and workflows.")
 
         # Discover and register tasks and workflows
@@ -95,24 +80,22 @@ class WorkflowApp(BaseModel):
 
         super().model_post_init(__context)
 
-    def get_chat_history(self) -> List[Any]:
-        """
-        Stub for fetching past conversation history. Override in subclasses.
-        """
-        logger.debug("Fetching chat history (default stub)")
-        return []
-
-    def _choose_llm_for(self, method: Callable) -> Optional[ChatClientType]:
+    def _choose_llm_for(self, method: Callable) -> Optional[ChatClientBase]:
         """
         Encapsulate LLM selection logic.
           1. Use per-task override if provided on decorator.
-          2. Else if marked as explicitly requiring an LLM, fall back to default app LLM.
+          2. Else if marked as explicitly requiring an LLM, fall back to default app LLM (if available).
           3. Otherwise, returns None.
         """
         per_task = getattr(method, "_task_llm", None)
         if per_task:
             return per_task
         if getattr(method, "_explicit_llm", False):
+            if self.llm is None:
+                logger.warning(
+                    f"Task '{getattr(method, '_task_name', getattr(method, '__name__', str(method)))}' requires an LLM "
+                    "but no default LLM is configured in WorkflowApp and no explicit LLM was provided."
+                )
             return self.llm
         return None
 
@@ -130,8 +113,170 @@ class WorkflowApp(BaseModel):
         logger.debug(f"Discovered tasks: {list(tasks)}")
         return tasks
 
+    def register_task(self, task_func: Callable, name: Optional[str] = None) -> None:
+        """
+        Manually register a @task-decorated function, similar to native Dapr pattern.
+
+        This allows explicit registration of tasks from other modules or packages.
+        The task will be registered immediately with the Dapr runtime.
+
+        Args:
+            task_func: The @task-decorated function to register
+            name: Optional custom name (defaults to function name or _task_name)
+
+        Example:
+            from tasks.my_tasks import generate_queries
+            wfapp.register_task(generate_queries)
+            wfapp.register_task(generate_queries, name="custom_name")
+        """
+        # Validate input
+        if not callable(task_func):
+            raise ValueError(
+                f"task_func must be callable, got {type(task_func)}: {task_func}"
+            )
+
+        if not getattr(task_func, "_is_task", False):
+            raise ValueError(
+                f"Function {getattr(task_func, '__name__', str(task_func))} is not decorated with @task"
+            )
+
+        task_name = name or getattr(
+            task_func, "_task_name", getattr(task_func, "__name__", "unknown_task")
+        )
+
+        # Check if already registered
+        if task_name in self.tasks:
+            logger.warning(f"Task '{task_name}' is already registered, skipping")
+            return
+
+        # Register immediately with Dapr runtime using existing registration logic
+        llm = self._choose_llm_for(task_func)
+        logger.debug(
+            f"Manually registering task '{task_name}' with llm={getattr(llm, '__class__', None)}"
+        )
+
+        kwargs = getattr(task_func, "_task_kwargs", {})
+        task_instance = WorkflowTask(
+            func=task_func,
+            description=getattr(task_func, "_task_description", None),
+            agent=getattr(task_func, "_task_agent", None),
+            llm=llm,
+            include_chat_history=getattr(
+                task_func, "_task_include_chat_history", False
+            ),
+            workflow_app=self,
+            **kwargs,
+        )
+
+        # Wrap for Dapr invocation
+        wrapped = self._make_task_wrapper(task_name, task_func, task_instance)
+        self.wf_runtime.register_activity(wrapped)
+        self.tasks[task_name] = wrapped
+
+    def register_tasks_from_module(
+        self, module_name_or_object: Union[str, Any]
+    ) -> None:
+        """
+        Register all @task-decorated functions from a specific module.
+
+        Args:
+            module_name_or_object: Module name string (e.g., "tasks.queries") or imported module object
+
+        Example:
+            # Using string name
+            wfapp.register_tasks_from_module("tasks.queries")
+
+            # Using imported module
+            import tasks.queries
+            wfapp.register_tasks_from_module(tasks.queries)
+
+            # Using from import
+            from tasks import queries
+            wfapp.register_tasks_from_module(queries)
+        """
+        try:
+            # Handle both string names and module objects
+            if isinstance(module_name_or_object, str):
+                import importlib
+
+                module = importlib.import_module(module_name_or_object)
+                module_name = module_name_or_object
+            else:
+                # Assume it's a module object
+                module = module_name_or_object
+                module_name = getattr(module, "__name__", str(module))
+
+            registered_count = 0
+            for name, fn in inspect.getmembers(module, inspect.isfunction):
+                if getattr(fn, "_is_task", False):
+                    task_name = getattr(fn, "_task_name", name)
+                    if task_name not in self.tasks:  # Skip if already registered
+                        self.register_task(fn)
+                        registered_count += 1
+
+            logger.info(
+                f"Registered {registered_count} tasks from module '{module_name}'"
+            )
+
+        except ImportError as e:
+            raise ImportError(f"Could not import module '{module_name_or_object}': {e}")
+        except Exception as e:
+            raise RuntimeError(
+                f"Error registering tasks from module '{module_name_or_object}': {e}"
+            )
+
+    def register_tasks_from_package(self, package_name: str) -> None:
+        """
+        Register all @task-decorated functions from all modules in a package.
+
+        Args:
+            package_name: Name of package to scan (e.g., "tasks")
+
+        Example:
+            wfapp.register_tasks_from_package("tasks")  # Scans tasks/*.py
+        """
+        try:
+            import importlib
+            import pkgutil
+
+            package = importlib.import_module(package_name)
+
+            # Collect all tasks first, then register using the original _register_tasks method
+            discovered_tasks: Dict[str, Callable] = {}
+
+            total_tasks = 0
+            for importer, modname, ispkg in pkgutil.iter_modules(package.__path__):
+                if not ispkg:  # Only scan modules, not sub-packages
+                    full_module_name = f"{package_name}.{modname}"
+                    try:
+                        module = importlib.import_module(full_module_name)
+
+                        for name, fn in inspect.getmembers(module, inspect.isfunction):
+                            if getattr(fn, "_is_task", False):
+                                task_name = getattr(fn, "_task_name", name)
+                                if (
+                                    task_name not in self.tasks
+                                ):  # Skip if already registered
+                                    discovered_tasks[task_name] = fn
+                                    total_tasks += 1
+
+                    except Exception as e:
+                        logger.warning(f"Failed to scan module {full_module_name}: {e}")
+
+            # Now register all discovered tasks using the original _register_tasks method
+            if discovered_tasks:
+                self._register_tasks(discovered_tasks)
+
+            logger.info(f"Registered {total_tasks} tasks from package '{package_name}'")
+        except ImportError as e:
+            raise ImportError(f"Could not import package '{package_name}': {e}")
+        except Exception as e:
+            raise RuntimeError(
+                f"Error registering tasks from package '{package_name}': {e}"
+            )
+
     def _register_tasks(self, tasks: Dict[str, Callable]) -> None:
-        """Register each discovered task with the Dapr runtime."""
+        """Register each discovered task with the Dapr runtime using direct registration."""
         for task_name, method in tasks.items():
             llm = self._choose_llm_for(method)
             logger.debug(
@@ -151,8 +296,10 @@ class WorkflowApp(BaseModel):
             )
             # Wrap for Dapr invocation
             wrapped = self._make_task_wrapper(task_name, method, task_instance)
-            activity_decorator = self.wf_runtime.activity(name=task_name)
-            self.tasks[task_name] = activity_decorator(wrapped)
+
+            # Use direct registration like official Dapr examples
+            self.wf_runtime.register_activity(wrapped)
+            self.tasks[task_name] = wrapped
 
     def _make_task_wrapper(
         self, task_name: str, method: Callable, task_instance: WorkflowTask
@@ -225,112 +372,6 @@ class WorkflowApp(BaseModel):
             decorator = self.wf_runtime.workflow(name=wf_name)
             self.workflows[wf_name] = decorator(make_wrapped(method))
 
-    def start_runtime(self):
-        """Idempotently start the Dapr workflow runtime."""
-        if not self.wf_runtime_is_running:
-            logger.info("Starting workflow runtime.")
-            self.wf_runtime.start()
-            self.wf_runtime_is_running = True
-        else:
-            logger.debug("Workflow runtime already running; skipping.")
-
-    def stop_runtime(self):
-        """Idempotently stop the Dapr workflow runtime."""
-        if self.wf_runtime_is_running:
-            logger.info("Stopping workflow runtime.")
-            self.wf_runtime.shutdown()
-            self.wf_runtime_is_running = False
-        else:
-            logger.debug("Workflow runtime already stopped; skipping.")
-
-    def register_agent(
-        self, store_name: str, store_key: str, agent_name: str, agent_metadata: dict
-    ) -> None:
-        """
-        Merges the existing data with the new data and updates the store.
-
-        Args:
-            store_name (str): The name of the Dapr state store component.
-            key (str): The key to update.
-            data (dict): The data to update the store with.
-        """
-        # retry the entire operation up to ten times sleeping 1 second between each
-        # TODO: rm the custom retry logic here and use the DaprClient retry_policy instead.
-        for attempt in range(1, 11):
-            try:
-                response: StateResponse = self.client.get_state(
-                    store_name=store_name, key=store_key
-                )
-                if not response.etag:
-                    # if there is no etag the following transaction won't work as expected
-                    # so we need to save an empty object with a strong consistency to force the etag to be created
-                    self.client.save_state(
-                        store_name=store_name,
-                        key=store_key,
-                        value=json.dumps({}),
-                        state_metadata={"contentType": "application/json"},
-                        options=StateOptions(
-                            concurrency=Concurrency.first_write,
-                            consistency=Consistency.strong,
-                        ),
-                    )
-                    # raise an exception to retry the entire operation
-                    raise Exception(f"No etag found for key: {store_key}")
-                existing_data = json.loads(response.data) if response.data else {}
-                if (agent_name, agent_metadata) in existing_data.items():
-                    logger.debug(f"agent {agent_name} already registered.")
-                    return None
-                agent_data = {agent_name: agent_metadata}
-                merged_data = {**existing_data, **agent_data}
-                logger.debug(f"merged data: {merged_data} etag: {response.etag}")
-                try:
-                    # using the transactional API to be able to later support the Dapr outbox pattern
-                    self.client.execute_state_transaction(
-                        store_name=store_name,
-                        operations=[
-                            TransactionalStateOperation(
-                                key=store_key,
-                                data=json.dumps(merged_data),
-                                etag=response.etag,
-                                operation_type=TransactionOperationType.upsert,
-                            )
-                        ],
-                        transactional_metadata={"contentType": "application/json"},
-                    )
-                except Exception as e:
-                    raise e
-                return None
-            except Exception as e:
-                logger.error(f"Error on transaction attempt: {attempt}: {e}")
-                logger.info("Sleeping for 1 second before retrying transaction...")
-                time.sleep(1)
-        raise Exception(
-            f"Failed to update state store key: {store_key} after 10 attempts."
-        )
-
-    def get_data_from_store(self, store_name: str, key: str) -> Optional[dict]:
-        """
-        Retrieves data from the Dapr state store using the given key.
-
-        Args:
-            store_name (str): The name of the Dapr state store component.
-            key (str): The key to fetch data from.
-
-        Returns:
-            Optional[dict]: the retrieved dictionary or None if not found.
-        """
-        try:
-            response: StateResponse = self.client.get_state(
-                store_name=store_name, key=key
-            )
-            data = response.data
-            return json.loads(data) if data else None
-        except Exception:
-            logger.warning(
-                f"Error retrieving data for key '{key}' from store '{store_name}'"
-            )
-            return None
-
     def resolve_task(self, task: Union[str, Callable]) -> Callable:
         """
         Resolves a registered task function by its name or decorated function.
@@ -347,7 +388,9 @@ class WorkflowApp(BaseModel):
         if isinstance(task, str):
             task_name = task
         elif callable(task):
-            task_name = getattr(task, "_task_name", task.__name__)
+            task_name = getattr(
+                task, "_task_name", getattr(task, "__name__", "unknown_task")
+            )
         else:
             raise ValueError(f"Invalid task reference: {task}")
 
@@ -373,7 +416,11 @@ class WorkflowApp(BaseModel):
         if isinstance(workflow, str):
             workflow_name = workflow  # Direct lookup by string name
         elif callable(workflow):
-            workflow_name = getattr(workflow, "_workflow_name", workflow.__name__)
+            workflow_name = getattr(
+                workflow,
+                "_workflow_name",
+                getattr(workflow, "__name__", "unknown_workflow"),
+            )
         else:
             raise ValueError(f"Invalid workflow reference: {workflow}")
 
@@ -383,6 +430,24 @@ class WorkflowApp(BaseModel):
 
         return workflow_func
 
+    def start_runtime(self):
+        """Idempotently start the Dapr workflow runtime."""
+        if not self.wf_runtime_is_running:
+            logger.info("Starting workflow runtime.")
+            self.wf_runtime.start()
+            self.wf_runtime_is_running = True
+        else:
+            logger.debug("Workflow runtime already running; skipping.")
+
+    def stop_runtime(self):
+        """Idempotently stop the Dapr workflow runtime."""
+        if self.wf_runtime_is_running:
+            logger.info("Stopping workflow runtime.")
+            self.wf_runtime.shutdown()
+            self.wf_runtime_is_running = False
+        else:
+            logger.debug("Workflow runtime already stopped; skipping.")
+
     def run_workflow(
         self, workflow: Union[str, Callable], input: Union[str, Dict[str, Any]] = None
     ) -> str:
@@ -400,10 +465,6 @@ class WorkflowApp(BaseModel):
             Exception: If workflow execution fails.
         """
         try:
-            # Start Workflow Runtime
-            if not self.wf_runtime_is_running:
-                self.start_runtime()
-
             # Generate unique instance ID
             instance_id = uuid.uuid4().hex
 
@@ -462,7 +523,7 @@ class WorkflowApp(BaseModel):
             logger.info(f"Monitoring workflow '{instance_id}'...")
 
             # Retrieve workflow state
-            state = await self.monitor_workflow_state(instance_id)
+            state: WorkflowState = await self.monitor_workflow_state(instance_id)
             if not state:
                 return  # Error already logged in monitor_workflow_state
 
@@ -566,8 +627,6 @@ class WorkflowApp(BaseModel):
             raise
         finally:
             logger.info(f"Finished workflow with Instance ID: {instance_id}.")
-            # Off-load the stop_runtime call as it may block.
-            await asyncio.to_thread(self.stop_runtime)
 
     def run_and_monitor_workflow_sync(
         self,
@@ -691,6 +750,11 @@ class WorkflowApp(BaseModel):
             logger.info(
                 f"Raising workflow event '{event_name}' for instance '{instance_id}'"
             )
+            # Ensure data is in a serializable format
+            if is_pydantic_model(type(data)):
+                # Convert Pydantic model to dict
+                data = data.model_dump()
+            # Raise the event using the Dapr workflow client with serialized data
             self.wf_client.raise_workflow_event(
                 instance_id=instance_id, event_name=event_name, data=data
             )
@@ -706,50 +770,6 @@ class WorkflowApp(BaseModel):
                 f"Failed to raise workflow event '{event_name}' for instance '{instance_id}': {str(e)}"
             )
 
-    def invoke_service(
-        self,
-        service: str,
-        method: str,
-        http_method: str = "POST",
-        input: Optional[Dict[str, Any]] = None,
-        timeout: Optional[int] = None,
-    ) -> Any:
-        """
-        Invokes an external service via Dapr.
-
-        Args:
-            service (str): The service name.
-            method (str): The method to call.
-            http_method (str, optional): The HTTP method (default: "POST").
-            input (Optional[Dict[str, Any]], optional): The request payload.
-            timeout (Optional[int], optional): Timeout in seconds.
-
-        Returns:
-            Any: The response from the service.
-
-        Raises:
-            Exception: If the invocation fails.
-        """
-        try:
-            resp = self.client.invoke_method(
-                app_id=service,
-                method_name=method,
-                http_verb=http_method,
-                data=json.dumps(input) if input else None,
-                timeout=timeout,
-            )
-            if resp.status_code != 200:
-                raise Exception(
-                    f"Error calling {service}.{method}: {resp.status_code}: {resp.text}"
-                )
-
-            agent_response = json.loads(resp.data.decode("utf-8"))
-            logger.info(f"Agent's Response: {agent_response}")
-            return agent_response
-        except Exception as e:
-            logger.error(f"Failed to invoke {service}.{method}: {e}")
-            raise e
-
     def when_all(self, tasks: List[dtask.Task[T]]) -> dtask.WhenAllTask[T]:
         """
         Waits for all given tasks to complete.
