@@ -1,5 +1,6 @@
 import asyncio
 from dapr_agents import tool, Agent
+from dapr_agents import OpenAIChatClient
 from dotenv import load_dotenv
 
 load_dotenv()
@@ -17,10 +18,13 @@ async def main():
         role="Weather Assistant",
         instructions=["Help users with weather information"],
         tools=[my_weather_func],
+        llm=OpenAIChatClient(model="gpt-3.5-turbo"),
     )
-
-    response = await weather_agent.run("What's the weather?")
-    print(response)
+    try:
+        response = await weather_agent.run("What's the weather?")
+        print(response)
+    except Exception as e:
+        print(f"Error: {e}")
 
 
 if __name__ == "__main__":
