@@ -1,6 +1,5 @@
 from dapr_agents.workflow import WorkflowApp, workflow, task
 from dapr.ext.workflow import DaprWorkflowContext
-
 from dotenv import load_dotenv
 
 load_dotenv()
@@ -31,6 +30,9 @@ def write_blog(outline: str) -> str:
 if __name__ == "__main__":
     wfapp = WorkflowApp()
 
-    results = wfapp.run_and_monitor_workflow_sync(analyze_topic, input="AI Agents")
-    if len(results) > 0:
-        print(f"Result: {results}")
+    try:
+        results = wfapp.run_and_monitor_workflow_sync(analyze_topic, input="AI Agents")
+        if len(results) > 0:
+            print(f"Result: {results}")
+    except Exception as e:
+        print(f"Error: {e}")
