@@ -1,5 +1,3 @@
-import os
-
 from dapr_agents.types.llm import AudioSpeechRequest
 from dapr_agents import OpenAIAudioClient
 from dotenv import load_dotenv
@@ -24,11 +22,8 @@ audio_bytes = client.create_speech(request=tts_request)
 # client.create_speech(request=tts_request, file_name=output_path)
 
 # Save the audio to an MP3 file
-output_path = "output_speech.mp3"
+output_path = "speech.mp3"
 with open(output_path, "wb") as audio_file:
     audio_file.write(audio_bytes)
 
 print(f"Audio saved to {output_path}")
-
-os.remove(output_path)
-print(f"File {output_path} has been deleted.")
